@@ -31,6 +31,7 @@ pub struct InitializeResponse {
     pub passes_session_recording_targeting: Option<bool>,
     pub session_recording_event_triggers: Option<HashMap<String, SessionReplayTrigger>>,
     pub session_recording_exposure_triggers: Option<HashMap<String, SessionReplayTrigger>>,
+    pub pa_hash: Option<String>,
 }
 
 impl InitializeResponse {
@@ -53,6 +54,7 @@ impl InitializeResponse {
             passes_session_recording_targeting: Default::default(),
             session_recording_event_triggers: Default::default(),
             session_recording_exposure_triggers: Default::default(),
+            pa_hash: user.get_hashed_private_attributes(),
         }
     }
 }
