@@ -1,7 +1,9 @@
 use ahash::AHashMap;
 use lazy_static::lazy_static;
+use parking_lot::{RwLock, RwLockWriteGuard};
 use std::any::Any;
-use std::sync::{Arc, RwLock, RwLockWriteGuard};
+use std::sync::Arc;
+use std::time::Duration;
 use uuid::Uuid;
 
 use crate::hashing::hash_one;
@@ -39,10 +41,10 @@ impl InstanceRegistry {
     }
 
     pub fn get<T: Send + Sync + 'static>(id: &u64) -> Option<Arc<T>> {
-        let registry = match REGISTRY.read() {
-            Ok(guard) => guard,
-            Err(e) => {
-                log_e!(TAG, "Failed to acquire read lock: {}", e);
+        let registry = match REGISTRY.try_read_for(Duration::from_secs(5)) {
+            Some(guard) => guard,
+            None => {
+                log_e!(TAG, "Failed to acquire read lock: Failed to lock REGISTRY");
                 return None;
             }
         };
@@ -63,10 +65,10 @@ impl InstanceRegistry {
     }
 
     pub fn get_raw(id: &u64) -> Option<Arc<dyn Any + Send + Sync>> {
-        let registry = match REGISTRY.read() {
-            Ok(guard) => guard,
-            Err(e) => {
-                log_e!(TAG, "Failed to acquire read lock: {}", e);
+        let registry = match REGISTRY.try_read_for(Duration::from_secs(5)) {
+            Some(guard) => guard,
+            None => {
+                log_e!(TAG, "Failed to acquire read lock: Failed to lock REGISTRY");
                 return None;
             }
         };
@@ -91,10 +93,10 @@ impl InstanceRegistry {
     }
 
     fn get_write_lock() -> Option<RwLockWriteGuard<'static, AHashMap<u64, AnyInstance>>> {
-        match REGISTRY.write() {
-            Ok(registry) => Some(registry),
-            Err(e) => {
-                log_e!(TAG, "Failed to acquire write lock: {}", e);
+        match REGISTRY.try_write_for(Duration::from_secs(5)) {
+            Some(registry) => Some(registry),
+            None => {
+                log_e!(TAG, "Failed to acquire write lock: Failed to lock REGISTRY");
                 None
             }
         }
